@@ -151,12 +151,12 @@ class DesignMode(TargetdbBase):
 
 
 class Design(TargetdbBase):
-    field = ForeignKeyField(column_name='field_pk',
-                            field='pk',
-                            model=Field,
-                            null=True,
-                            backref="designs")
-    exposure = IntegerField(null=True)
+    # field = ForeignKeyField(column_name='field_pk',
+    #                         field='pk',
+    #                         model=Field,
+    #                         null=True,
+    #                         backref="designs")
+    # exposure = IntegerField(null=True)
     design_id = AutoField()
     design_mode = ForeignKeyField(column_name='design_mode_label',
                                   field='label',
@@ -165,11 +165,27 @@ class Design(TargetdbBase):
     mugatu_version = TextField()
     run_on = DateTimeField(default=datetime.datetime.now())
     assignment_hash = UUIDField()
+    # field_exposure = IntegerField()
 
     class Meta:
         table_name = 'design'
 
 
+class DesignToField(TargetdbBase):
+    pk = AutoField()
+    design = ForeignKeyField(model=Design,
+                             column_name='design_id',
+                             field='design_id',)
+    field = ForeignKeyField(column_name='field_pk',
+                            field='pk',
+                            model=Field)
+    exposure = IntegerField()
+    field_exposure = IntegerField()
+
+    class Meta:
+        table_name = 'design_to_field'
+
+
 class DesignModeCheckResults(TargetdbBase):
     pk = IntegerField(null=False, primary_key=True)
     design = ForeignKeyField(Design,
@@ -317,6 +333,7 @@ class CartonToTarget(TargetdbBase):
                                  field='pk')
     delta_ra = DoubleField()
     delta_dec = DoubleField()
+    can_offset = BooleanField(default=False)
     inertial = BooleanField()
 
     class Meta:
